@@ -35,22 +35,24 @@
 #include <string.h>
 
 #include "glib.h"
-#include "galias.h"
 #include "gdatasetprivate.h"
+#include "galias.h"
 
 
 /* --- defines --- */
 #define	G_QUARK_BLOCK_SIZE			(512)
-#define	G_DATA_CACHE_MAX			(512)
 
-/* datalist pointer modifications have to be done with the g_dataset_global mutex held */
+/* datalist pointer accesses have to be carried out atomically */
 #define G_DATALIST_GET_POINTER(datalist)						\
-  ((GData*) ((gsize) *(datalist) & ~(gsize) G_DATALIST_FLAGS_MASK))
-#define G_DATALIST_SET_POINTER(datalist, pointer) G_STMT_START {			\
-  *(datalist) = (GData*) (G_DATALIST_GET_FLAGS (datalist) |				\
-			  (gsize) pointer);						\
-} G_STMT_END
+  ((GData*) ((gsize) g_atomic_pointer_get ((gpointer*) datalist) & ~(gsize) G_DATALIST_FLAGS_MASK))
 
+#define G_DATALIST_SET_POINTER(datalist, pointer)       G_STMT_START {                  \
+  gpointer _oldv, _newv;                                                                \
+  do {                                                                                  \
+    _oldv = g_atomic_pointer_get (datalist);                                            \
+    _newv = (gpointer) (((gsize) _oldv & G_DATALIST_FLAGS_MASK) | (gsize) pointer);     \
+  } while (!g_atomic_pointer_compare_and_exchange ((void**) datalist, _oldv, _newv));   \
+} G_STMT_END
 
 /* --- structures --- */
 typedef struct _GDataset GDataset;
@@ -87,9 +89,6 @@ G_LOCK_DEFINE_STATIC (g_dataset_global);
 static GHashTable   *g_dataset_location_ht = NULL;
 static GDataset     *g_dataset_cached = NULL; /* should this be
 						 threadspecific? */
-static GData	    *g_data_cache = NULL;
-static guint	     g_data_cache_length = 0;
-
 G_LOCK_DEFINE_STATIC (g_quark_global);
 static GHashTable   *g_quark_ht = NULL;
 static gchar       **g_quarks = NULL;
@@ -122,14 +121,7 @@ g_datalist_clear_i (GData **datalist)
 	  G_LOCK (g_dataset_global);
 	}
       
-      if (g_data_cache_length < G_DATA_CACHE_MAX)
-	{
-	  prev->next = g_data_cache;
-	  g_data_cache = prev;
-	  g_data_cache_length++;
-	}
-      else
-	g_slice_free (GData, prev);
+      g_slice_free (GData, prev);
     }
 }
 
@@ -253,14 +245,7 @@ g_data_set_internal (GData	  **datalist,
 	      else
 		ret_data = list->data;
 	      
-	      if (g_data_cache_length < G_DATA_CACHE_MAX)
-		{
-		  list->next = g_data_cache;
-		  g_data_cache = list;
-		  g_data_cache_length++;
-		}
-	      else
-		g_slice_free (GData, list);
+              g_slice_free (GData, list);
 	      
 	      return ret_data;
 	    }
@@ -304,14 +289,7 @@ g_data_set_internal (GData	  **datalist,
 	  list = list->next;
 	}
       
-      if (g_data_cache)
-	{
-	  list = g_data_cache;
-	  g_data_cache = list->next;
-	  g_data_cache_length--;
-	}
-      else
-	list = g_slice_new (GData);
+      list = g_slice_new (GData);
       list->next = G_DATALIST_GET_POINTER (datalist);
       list->id = key_id;
       list->data = data;
@@ -526,8 +504,8 @@ void
 g_datalist_init (GData **datalist)
 {
   g_return_if_fail (datalist != NULL);
-  
-  *datalist = NULL;
+
+  g_atomic_pointer_set ((gpointer*) datalist, NULL);
 }
 
 /**
@@ -545,17 +523,23 @@ g_datalist_init (GData **datalist)
  * not generally useful except in circumstances where space
  * is very tight. (It is used in the base #GObject type, for
  * example.)
+ *
+ * Since: 2.8
  **/
 void
 g_datalist_set_flags (GData **datalist,
 		      guint   flags)
 {
+  gpointer oldvalue;
   g_return_if_fail (datalist != NULL);
   g_return_if_fail ((flags & ~G_DATALIST_FLAGS_MASK) == 0);
-
-  G_LOCK (g_dataset_global);
-  *datalist = (GData*) (flags | (gsize) *datalist);
-  G_UNLOCK (g_dataset_global);
+  
+  do
+    {
+      oldvalue = g_atomic_pointer_get (datalist);
+    }
+  while (!g_atomic_pointer_compare_and_exchange ((void**) datalist, oldvalue,
+                                                 (gpointer) ((gsize) oldvalue | flags)));
 }
 
 /**
@@ -568,17 +552,23 @@ g_datalist_set_flags (GData **datalist,
  *   an error.
  * 
  * Turns off flag values for a data list. See g_datalist_unset_flags()
+ *
+ * Since: 2.8
  **/
 void
 g_datalist_unset_flags (GData **datalist,
 			guint   flags)
 {
+  gpointer oldvalue;
   g_return_if_fail (datalist != NULL);
   g_return_if_fail ((flags & ~G_DATALIST_FLAGS_MASK) == 0);
-
-  G_LOCK (g_dataset_global);
-  *datalist = (GData*) (~(gsize) flags & (gsize) *datalist);
-  G_UNLOCK (g_dataset_global);
+  
+  do
+    {
+      oldvalue = g_atomic_pointer_get (datalist);
+    }
+  while (!g_atomic_pointer_compare_and_exchange ((void**) datalist, oldvalue,
+                                                 (gpointer) ((gsize) oldvalue & ~(gsize) flags)));
 }
 
 /**
@@ -589,12 +579,14 @@ g_datalist_unset_flags (GData **datalist,
  * See g_datalist_set_flags().
  * 
  * Return value: the flags of the datalist
+ *
+ * Since: 2.8
  **/
 guint
 g_datalist_get_flags (GData **datalist)
 {
   g_return_val_if_fail (datalist != NULL, 0);
-
+  
   return G_DATALIST_GET_FLAGS (datalist); /* atomic macro */
 }
 
@@ -622,6 +614,22 @@ g_quark_try_string (const gchar *string)
   return quark;
 }
 
+/* HOLDS: g_quark_global_lock */
+static inline GQuark
+g_quark_from_string_internal (const gchar *string, 
+			      gboolean     duplicate)
+{
+  GQuark quark = 0;
+  
+  if (g_quark_ht)
+    quark = GPOINTER_TO_UINT (g_hash_table_lookup (g_quark_ht, string));
+  
+  if (!quark)
+    quark = g_quark_new (duplicate ? g_strdup (string) : (gchar *)string);
+  
+  return quark;
+}
+
 GQuark
 g_quark_from_string (const gchar *string)
 {
@@ -630,16 +638,7 @@ g_quark_from_string (const gchar *string)
   g_return_val_if_fail (string != NULL, 0);
   
   G_LOCK (g_quark_global);
-  if (g_quark_ht)
-    quark = (gulong) g_hash_table_lookup (g_quark_ht, string);
-  else
-    {
-      g_quark_ht = g_hash_table_new (g_str_hash, g_str_equal);
-      quark = 0;
-    }
-  
-  if (!quark)
-    quark = g_quark_new (g_strdup (string));
+  quark = g_quark_from_string_internal (string, TRUE);
   G_UNLOCK (g_quark_global);
   
   return quark;
@@ -653,18 +652,9 @@ g_quark_from_static_string (const gchar *string)
   g_return_val_if_fail (string != NULL, 0);
   
   G_LOCK (g_quark_global);
-  if (g_quark_ht)
-    quark = (gulong) g_hash_table_lookup (g_quark_ht, string);
-  else
-    {
-      g_quark_ht = g_hash_table_new (g_str_hash, g_str_equal);
-      quark = 0;
-    }
-
-  if (!quark)
-    quark = g_quark_new ((gchar*) string);
+  quark = g_quark_from_string_internal (string, FALSE);
   G_UNLOCK (g_quark_global);
- 
+
   return quark;
 }
 
@@ -672,9 +662,10 @@ G_CONST_RETURN gchar*
 g_quark_to_string (GQuark quark)
 {
   gchar* result = NULL;
+
   G_LOCK (g_quark_global);
-  if (quark > 0 && quark <= g_quark_seq_id)
-    result = g_quarks[quark - 1];
+  if (quark < g_quark_seq_id)
+    result = g_quarks[quark];
   G_UNLOCK (g_quark_global);
 
   return result;
@@ -688,10 +679,15 @@ g_quark_new (gchar *string)
   
   if (g_quark_seq_id % G_QUARK_BLOCK_SIZE == 0)
     g_quarks = g_renew (gchar*, g_quarks, g_quark_seq_id + G_QUARK_BLOCK_SIZE);
-  
-  g_quarks[g_quark_seq_id] = string;
-  g_quark_seq_id++;
-  quark = g_quark_seq_id;
+  if (!g_quark_ht)
+    {
+      g_assert (g_quark_seq_id == 0);
+      g_quark_ht = g_hash_table_new (g_str_hash, g_str_equal);
+      g_quarks[g_quark_seq_id++] = NULL;
+    }
+
+  quark = g_quark_seq_id++;
+  g_quarks[quark] = string;
   g_hash_table_insert (g_quark_ht, string, GUINT_TO_POINTER (quark));
   
   return quark;
@@ -711,7 +707,18 @@ g_quark_new (gchar *string)
 G_CONST_RETURN gchar*
 g_intern_string (const gchar *string)
 {
-   return string ? g_quark_to_string (g_quark_from_string (string)) : NULL;
+  const gchar *result;
+  GQuark quark;
+
+  if (!string)
+    return NULL;
+
+  G_LOCK (g_quark_global);
+  quark = g_quark_from_string_internal (string, TRUE);
+  result = g_quarks[quark];
+  G_UNLOCK (g_quark_global);
+
+  return result;
 }
 
 /**
@@ -730,7 +737,18 @@ g_intern_string (const gchar *string)
 G_CONST_RETURN gchar*
 g_intern_static_string (const gchar *string)
 {
-   return string ? g_quark_to_string (g_quark_from_static_string (string)) : NULL;
+  GQuark quark;
+  const gchar *result;
+
+  if (!string)
+    return NULL;
+
+  G_LOCK (g_quark_global);
+  quark = g_quark_from_string_internal (string, FALSE);
+  result = g_quarks[quark];
+  G_UNLOCK (g_quark_global);
+
+  return result;
 }
 
 
